@@ -1,5 +1,7 @@
 from typing import Callable, Dict, List, Tuple, Union
 
+from jycm.common import (EVENT_DICT_ADD, EVENT_DICT_REMOVE, EVENT_LIST_ADD, EVENT_LIST_REMOVE, EVENT_PAIR,
+                         EVENT_VALUE_CHANGE, PLACE_HOLDER_NON_EXIST)
 from jycm.helper import make_json_path_key
 from jycm.km_matcher import KMMatcher
 from jycm.operator import BaseOperator
@@ -14,9 +16,11 @@ class TreeLevel:
         left_path: left json path
         right_path: right json path
         up: the parent TreeLevel
+        diff: a simple way to inject custom operators ; default None
     """
 
-    def __init__(self, left, right, left_path: List, right_path: List, up: Union['TreeLevel', None]):
+    def __init__(self, left, right, left_path: List, right_path: List, up: Union['TreeLevel', None],
+                 diff: Union[Callable[['TreeLevel', bool], Tuple[bool, float]], None] = None):
         """Init method for TreeLevel
 
         """
@@ -28,6 +32,11 @@ class TreeLevel:
 
         self.up: TreeLevel = up
 
+        if diff is not None:
+            self.diff = lambda _drill: diff(self, _drill)
+        else:
+            self.diff = None
+
     def get_type(self) -> type:
         """Get the type of this level
 
@@ -60,7 +69,9 @@ class TreeLevel:
             The left path of this level (matching will be concerned on left mainly)
 
         """
-        return make_json_path_key(self.left_path)
+        if self.left != PLACE_HOLDER_NON_EXIST:
+            return make_json_path_key(self.left_path)
+        return make_json_path_key(self.right_path)
 
     def get_key(self):
         """ Get the key of this level
@@ -118,15 +129,6 @@ class Record:
         }
 
 
-EVENT_PAIR = "pairs"
-EVENT_DICT_REMOVE = "dict:remove"
-EVENT_DICT_ADD = "dict:add"
-EVENT_LIST_REMOVE = "list:remove"
-EVENT_LIST_ADD = "list:add"
-
-PLACE_HOLDER_NON_EXIST = "__NON_EXIST__"
-
-
 def gather_serial_pair(target_index, indices, list_, container):
     """Match parts between LCS index
 
@@ -210,6 +212,8 @@ class YouchamaJsonDiffer:
 
         self.ignore_order_func: Callable[[TreeLevel, bool], bool] = ignore_order_func
 
+        self.event_pair_dict: Dict[str, bool] = {}
+
     def report_pair(self, level: TreeLevel):
         """Report pair of json path
 
@@ -219,7 +223,13 @@ class YouchamaJsonDiffer:
             level: TreeLevel
 
         """
-        if make_json_path_key(level.left_path) != make_json_path_key(level.right_path):
+        left_key = make_json_path_key(level.left_path)
+        right_key = make_json_path_key(level.right_path)
+        unique_key = f"{left_key}__@__{right_key}"
+
+        if left_key != right_key and unique_key not in self.event_pair_dict:
+            self.event_pair_dict[unique_key] = True
+
             self.records[EVENT_PAIR].append(Record(
                 event=EVENT_PAIR, level=level, info={}
             ))
@@ -510,7 +520,7 @@ class YouchamaJsonDiffer:
                     left=PLACE_HOLDER_NON_EXIST,
                     right=level.right[i],
                     left_path=[],
-                    right_path=[*level.left_path, i],
+                    right_path=[*level.right_path, i],
                     up=level
                 ))
 
@@ -622,7 +632,7 @@ class YouchamaJsonDiffer:
         for pair in pair_list:
             # still can be different under not drill
             self.diff_level(pair.level, False)
-            self.report_pair(level)
+            self.report_pair(pair.level)
             matched_left_index.append(pair.left_index)
             matched_right_index.append(pair.right_index)
 
@@ -667,6 +677,7 @@ class YouchamaJsonDiffer:
                     pair_list.append(ListItemPair(value=new_level, left_index=li, right_index=ri))
                     matched_right[ri] = True
                     matched_left[li] = True
+                    break
 
         if not drill:
             # only in the report phase
@@ -699,6 +710,30 @@ class YouchamaJsonDiffer:
         if self.debug:
             print(f"[compare_dict>>>] {level}")
 
+        ref = self
+
+        def __dict_remove_diff(_level: 'TreeLevel', _drill: bool):
+            if not _drill:
+                ref.report(EVENT_DICT_REMOVE, TreeLevel(
+                    left=_level.left,
+                    right=PLACE_HOLDER_NON_EXIST,
+                    left_path=[*_level.left_path],
+                    right_path=[],
+                    up=level
+                ))
+            return True, 0
+
+        def __dict_add_diff(_level: 'TreeLevel', _drill: bool):
+            if not _drill:
+                ref.report(EVENT_DICT_ADD, TreeLevel(
+                    left=PLACE_HOLDER_NON_EXIST,
+                    right=_level.right,
+                    left_path=[],
+                    right_path=[*_level.right_path],
+                    up=level
+                ))
+            return True, 0
+
         for k in all_keys:
             if k in level.right and k in level.left:
                 _score = self.diff_level(TreeLevel(
@@ -718,25 +753,30 @@ class YouchamaJsonDiffer:
                     self.report_pair(level)
                 continue
 
-            if k in level.left and not drill:
-                self.report(EVENT_DICT_REMOVE, TreeLevel(
+            if k in level.left:
+                _score = self.diff_level(TreeLevel(
                     left=level.left[k],
                     right=PLACE_HOLDER_NON_EXIST,
                     left_path=[*level.left_path, k],
                     right_path=[],
-                    up=level
-                ))
+                    up=level,
+                    diff=__dict_remove_diff
+                ), drill=drill)
+                score += _score
                 continue
 
-            if k in level.right and not drill:
-                self.report(EVENT_DICT_ADD, TreeLevel(
+            if k in level.right:
+                _score = self.diff_level(TreeLevel(
                     left=PLACE_HOLDER_NON_EXIST,
                     right=level.right[k],
                     left_path=[],
                     right_path=[*level.right_path, k],
-                    up=level
-                ))
+                    up=level,
+                    diff=__dict_add_diff
+                ), drill=drill)
+                score += _score
                 continue
+
         if len(all_keys) == 0:
             return 1
         return score / len(all_keys)
@@ -757,7 +797,7 @@ class YouchamaJsonDiffer:
         if level.left != level.right:
             # 非演习
             if not drill:
-                self.report("value_changes", level, {
+                self.report(EVENT_VALUE_CHANGE, level, {
                     "old": level.left,
                     "new": level.right
                 })
@@ -795,23 +835,40 @@ class YouchamaJsonDiffer:
             A score between 0~1 to describe how similar **level.left** and **level.right** are.
 
         """
-        skip, score = self.use_custom_operators(level, drill)
-
-        if skip:
-            return score
 
         try:
-            level_type = level.get_type()
-        except DifferentTypeException:
-            return 0
 
-        if level_type == list:
-            return self.compare_list(level, drill)
+            if not drill:
+                # just report
+                self.report_pair(level)
 
-        if level_type == dict:
-            return self.compare_dict(level, drill)
+            skip, score = self.use_custom_operators(level, drill)
 
-        return self.compare_primitive(level, drill)
+            if skip:
+                return score
+
+            if level.diff is not None:
+                skip, score = level.diff(drill)
+                if skip:
+                    return score
+
+            try:
+                level_type = level.get_type()
+            except DifferentTypeException:
+                # just use compare_primitive to report a value_change
+                return self.compare_primitive(level, drill)
+
+            if level_type == list:
+                return self.compare_list(level, drill)
+
+            if level_type == dict:
+                return self.compare_dict(level, drill)
+
+            return self.compare_primitive(level, drill)
+        except DiffLevelException as e:
+            raise e
+        except Exception as e:
+            raise DiffLevelException(f"Error {e} [drill={drill}] when compare [{level}]")
 
     def diff_level(self, level: TreeLevel, drill: bool) -> float:
         """Diff level function\
